﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;

namespace Inedo.Consolation
{
    internal sealed class ConsoleCommand
    {
        private MethodInfo method;

        public ConsoleCommand(MethodInfo method)
        {
            this.method = method;
            this.Options = method.GetParameters()
                .Select(p => new CommandConsoleOption(p))
                .ToList();
        }

        public string Name
        {
            get { return this.method.Name.Replace('_', '-'); }
        }
        public IEnumerable<string> Aliases
        {
            get
            {
                return this.method
                    .GetCustomAttributes(typeof(ConsoleAliasAttribute), true)
                    .Cast<ConsoleAliasAttribute>()
                    .Select(a => a.Alias);
            }
        }
        public string Description
        {
            get
            {
                var attr = (DescriptionAttribute)Attribute.GetCustomAttribute(this.method, typeof(DescriptionAttribute));
                if (attr != null)
                    return attr.Description;
                else
                    return null;
            }
        }
        public IReadOnlyList<CommandConsoleOption> Options { get; }
        public bool IsBrowsable => ((BrowsableAttribute)Attribute.GetCustomAttribute(this.method, typeof(BrowsableAttribute)))?.Browsable ?? true;

        public int Invoke()
        {
            var requiredValueThatIsNotInclused = this.Options
                .FirstOrDefault(o => o.CurrentValue == null && o.DefaultValue == null && o.IsRequired);

            if (requiredValueThatIsNotInclused != null)
                throw new ConsoleArgumentException(string.Format("Missing required value for \"{0}\"", requiredValueThatIsNotInclused.Name));

            var argValues = this.Options
                .Select(o => o.CurrentValue)
                .ToArray();

            var result = this.method.Invoke(null, argValues);
            if (result is int)
                return (int)result;
            else
                return 0;
        }
        public CommandConsoleOption GetPositionalOption(int position)
        {
            int index = 0;
            foreach (var option in this.Options)
            {
                if (option.IsPositional)
                {
                    if (index == position)
                        return option;
                    else
                        index++;
                }
            }

            return null;
        }
    }
}
